@@ -18,7 +18,7 @@
 
 /*
  * The functioning of the download architecture, as described by Philip
- * on 28 May 2001 and updated on 28 June 2001:
+ * on 28 May 2001 and updated on 28 June 2001, and again on 05 Jan 2005:
  * 
  * When mozilla runs into a file it cannot render internally or that it
  * does not have a plugin for, it calls the
@@ -36,7 +36,10 @@
  * mozilla-the-browser. However, for dealing with helper apps, we do not
  * use the mozilla backend at all. This is because we want to use the
  * gnome-vfs database to retrieve helper-app info, rather than the
- * mozilla helper app database.
+ * mozilla helper app database. Note that mozilla now uses gnome-vfs for
+ * it's backend, so in theory, we don't need this step. In reality, I 
+ * remain suspicious of gnome-vfs's mime database and frankly don't trust
+ * any other code to work around its stupidities correctly :-)
  * 
  * How it works:
  * 
@@ -46,40 +49,30 @@
  * nsIHelperAppDialog.
  * 
  * b) In our implementation of Show() we first compare the passed mime
- * type to galeon's mime list. If the mime type is in the list, we then
- * lookup the Action associated with the mime type. Currently, the
- * possible mime-actions are:
+ * type to gnome-vfs's mime list. If any helpers are available, we build
+ * a list of them and offer the user the ability to open the file or save
+ * it to disk. If there are no helpers, we just offer the option of saving
+ * or cancelling the downlod.
  * 
- * Save to Disk
- * Run with Helper App
- * Ask User
- * 
- * The default action is Ask User, and if the mime-type is not in our
- * list, this is what will be assumed.
- * 
- * c) If Ask User is the chosen action, a dialog will be shown to the
- * user allowing the user to choose from the other two possible actions
- * as well as a checkbox to let the user set the default action to the
- * chosen action for the future.
- * 
- * d-1) The "Save to Disk" action. We first check galeon preferences to
- * see if the user wants to use the built-in mozilla downloader, gtm or
- * a command-line executed downloader.
+ * c-1) The "Save to Disk" action. We first check galeon preferences to
+ * see if the user wants to use the built-in mozilla downloader, or a
+ * command-line executed downloader. We still have our gtm code but until
+ * gtm is updated to use bonobo-activation, we can't use it effectively.
  *
- * d-2a) The built-in downloader.  This action is handled by the mozilla
+ * c-2a) The built-in downloader.  This action is handled by the mozilla
  * backend. Our nsIHelperAppDialog does the same thing that the
  * mozilla-the-browser one does, which is to call the SaveToDisk() method
  * of nsIExternalHelperAppService. This in turn calls the
  * PromptForSaveToFile() method of nsIHelperAppDialog putting the ball
  * back in our court.
  * 
- * d-2b) Now, if galeon is configured to always ask for a download
+ * c-2b) Now, if galeon is configured to always ask for a download
  * directory, it will pop up a file selector so that the user can select
  * the directory and filename to save the file to.  Otherwise, it will
  * use galeon's default download directory and proceed without
  * interaction.
  * 
- * d-2c) When PromptForSaveToFile() returns, nsIExternalHelperAppService
+ * c-2c) When PromptForSaveToFile() returns, nsIExternalHelperAppService
  * will then call the ShowProgressDialog() method of
  * nsIHelperAppDialog. This progress dialog, obviously, tracks the
  * progress of the download. It is worth noting that mozilla starts the
@@ -95,34 +88,26 @@
  * the download starts early means that the file is actually downloaded
  * to a temp file and only at the end is it moved to it's final location.
  * 
- * d-3a) The two external downloader options.  These options are
+ * c-3a) The two external downloader options.  These options are
  * handled completely by galeon. The first thing that we do is call the
  * Cancel() method of nsIExternalHelperAppService to cancel the mozilla
  * download. We then pass the url to our own LaunchExternalDownloader()
  * method. This method will ask for a download directory as appropriate
  * as with the "Save to disk" action.
  * 
- * d-3b) Finally, depending on whether GTM or a command line handler was
+ * c-3b) Finally, depending on whether GTM or a command line handler was
  * selected in prefs, the external handler will be called with the url
  * passed and the directory selected.
  * 
- * e-1) The "Run with Helper App" action.  This action is currently only
- * working with a minimal implementation.  First, we explicitly call
- * ShowProgressDialog() so the user knows that the file is being
- * downloaded. We also need this so that we only run the helper after the
- * file is completely downloaded. The file will download to temp location
- * that it would be moved from if the action was "Save to Disk".  We have
- * to call ShowProgressDialog() ourselves because we are not using
- * mozilla's helper mechanism which would usually make the call for us.
- * 
- * e-2) If there is a default helper app in our mime database and alwaysAsk
- * is false, galeon will run the default helper automatically. Otherwise it
- * will pop up a helper chooser dialog which lists the helpers that gnome-vfs
- * knows about as well as providing a GnomeFileEntry to allow the user to
- * select and arbitrary application. The default value of the GnomeFileEntry
- * is the helper stored in our database if one exits.
- * 
- * f) General notes.  We cannot use this infrastructure to override
+ * d-1) The "Run with Helper App" action. This implementation is slightly
+ * subversive with respect to the mozilla helper db but allows us to avoid
+ * inventing an information passing mechanism between ContentHandler and
+ * ProgressListener. Rather, we populate the mozilla MIME Info struct with
+ * the handler we retrieved from gnome-vfs but explicitly tell mozilla not
+ * to launch the helper. The ProgressListener retrieves this info when the
+ * download completes and then runs the helper.
+ *
+ * e) General notes.  We cannot use this infrastructure to override
  * native mozilla types. mozilla will attempt to render these types and
  * never call out to us. We are at the end of the chain as the handler of
  * last resort, so native and plugin types will never reach us. This also
@@ -132,26 +117,16 @@
  * 
  * Despite the apparent user-side similarity with explicit downloads by
  * a shift-click or context-menu item, there is actually none at all.
- * Explicit downloads are handled by the nsIStreamTransfer manager which
- * we use as is. Currently the progress dialog for the stream transfer
- * manager is un-overridable, so it appears in XUL. This will change in
- * due course.
+ * Explicit downloads are handled by nsIWebBrowserPersist. We reuse the
+ * progress dialog for these but nothing in ContentHandler is involved.
  * 
- * Matt would like the modifiy the progress dialog so each file currently
- * being downloaded becomes a clist entry in a master dialog rather than
- * causing a separate progress dialog. a lot of progress dialogs gets
- * really messy.
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-extern "C" {
-#include "libgnomevfs/gnome-vfs-mime-handlers.h"
-}
 
-#include "galeon-embed-shell.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
 #include "gul-download.h"
@@ -160,50 +135,46 @@ extern "C" {
 #include "gul-general.h"
 #include "gul-gui.h"
 #include "galeon-embed-utils.h"
-#include "ProgressListener.h"
-#include "ContentHandler.h"
-#include "disclosure-widget.h"
+#include "gul-file-chooser.h"
+#include "galeon-debug.h"
+#include "hig-alert.h"
 
-#include <gtk/gtkentry.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkprogress.h>
-#include <gtk/gtkoptionmenu.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkliststore.h>
 #include <gtk/gtkwindow.h>
-#include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtkcellrenderertext.h>
-#include <libgnome/gnome-exec.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnome/gnome-config.h>
-#include <libgnome/gnome-util.h>
+#include <gtk/gtkexpander.h>
+#include <gtk/gtkmain.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
-#include "FilePicker.h"
-#include "MozillaPrivate.h"
+#include "AutoJSContextStack.h"
+#include "ContentHandler.h"
+#include "GaleonUtils.h"   /* GaleonUtils::FindGtkParent */
+#include "ProgressListener.h" /* for BuildDownloadPath */
+#include "GulString.h"
 
-#include "nsCRT.h"
-#include "nsCOMPtr.h"
-#include "nsISupportsArray.h"
-#include "nsIServiceManager.h"
-#include "nsWeakReference.h"
-
-#include "nsString.h"
-#include "nsIURI.h"
-#include "nsIURL.h"
-#include "nsIMIMEInfo.h"
-#include "nsIChannel.h"
-#include "nsIFTPChannel.h"
-#include "nsILocalFile.h"
-#include "nsIPrefService.h"
-#include "nsIDOMWindow.h"
-#include "nsIDOMWindowInternal.h"
+#include <nsIPromptService.h>
+#include <nsIWebProgressListener.h>
+#include <nsCOMPtr.h>
+#define MOZILLA_INTERNAL_API
+#include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
+#include <nsIInterfaceRequestorUtils.h>
+#include <nsIURI.h>
+#include <nsIURL.h>
+#include <nsIMIMEInfo.h>
+#include <nsILocalFile.h>
+#include <nsIDOMWindow.h>
+#include <nsIExternalHelperAppService.h>
+#include <nsCExternalHandlerService.h>
+#include <nsMemory.h>
+#include <nsNetError.h>
 
 class GContentHandler;
-class GDownloadProgressListener;
 struct MimeAskActionDialog;
-struct HelperAppChooserDialog;
 
 extern "C"
 void mime_ask_dialog_save_clicked_cb (GtkButton *button,
@@ -226,17 +197,12 @@ struct MimeAskActionDialog
 			    const char *aMimeType);
 	~MimeAskActionDialog();
 
-	GContentHandler *mContentHandler;
-	GladeXML *mGXml;
+	GContentHandler* mContentHandler;
 	GtkWidget *mDialogWidget;
 	GtkWidget *mParent;
-	GtkWidget *mAppMenu;
-	GtkWidget *mToggleButton;
 	GtkWidget *mListView;
-	GtkWidget *mScrolledView;
 
 	GList *mApps;
-	GtkListStore *mListStore;
 
 	enum
 	{
@@ -248,23 +214,57 @@ struct MimeAskActionDialog
 	static const int NUM_APPLICATIONS = 4; 
 };
 
+
+/* ------------------------------------------------------------
+ * Mini progress listener so that we can listen for errors
+ * while downloading */
+class MiniProgressListener
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+	: public nsIWebProgressListener2
+#else
+	: public nsIWebProgressListener
+#endif
+{
+public:
+	MiniProgressListener(MimeAskActionDialog *dialog) : 
+		mDialog( dialog ) 
+	{ LOG ("MiniProgresListener ctor(%p)", this ); };
+
+	~MiniProgressListener() 
+	{ LOG ("MiniProgresListener dtor(%p)", this ); };
+
+ 	NS_DECL_ISUPPORTS
+	NS_DECL_NSIWEBPROGRESSLISTENER
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+	NS_DECL_NSIWEBPROGRESSLISTENER2
+#endif
+	
+	MimeAskActionDialog *mDialog;
+};
+
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+NS_IMPL_ISUPPORTS2(MiniProgressListener, nsIWebProgressListener2,
+		   nsIWebProgressListener)
+#else
+NS_IMPL_ISUPPORTS1(MiniProgressListener,
+		   nsIWebProgressListener)
+#endif
+
+// ------------------------------------------------------------
+
 /* Implementation file */
 NS_IMPL_ISUPPORTS1(GContentHandler, nsIHelperAppLauncherDialog)
 
-GContentHandler::GContentHandler() : mUri(nsnull),
-				     mMimeType(nsnull),
-				     mDownloadCanceled(PR_FALSE),
-				     mHelperProgress(PR_FALSE)
+GContentHandler::GContentHandler() : mUri(nsnull)
 {
-	NS_INIT_ISUPPORTS();
 	/* member initializers and constructor code */
+	LOG ("GContentHandler ctor(%p)", this );
 }
 
 GContentHandler::~GContentHandler()
 {
 	/* destructor code */
-	g_free (mUri);
-	g_free (mMimeType);
+	LOG ("GContentHandler dtor(%p)", this );
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -273,11 +273,11 @@ GContentHandler::~GContentHandler()
 
 /* void show (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
-				    nsISupports *aContext
-#if MOZILLA_SNAPSHOT > 9
-, PRBool aForced)
+				    nsISupports *aContext,
+#ifdef HAVE_NSIHELPERAPPLAUNCHERDIALOG_SHOW_REASON
+				    PRUint32 aReason)
 #else
-)
+				    PRBool aForced)
 #endif
 {
 	/* aForced reflects if the content being sent is normally viewable
@@ -296,140 +296,69 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 
 /* nsILocalFile promptForSaveToFile (in nsISupports aWindowContext, in wstring aDefaultFile, in wstring aSuggestedFileExtension); */
 NS_IMETHODIMP GContentHandler::PromptForSaveToFile(
-#if MOZILLA_SNAPSHOT > 10
 						   nsIHelperAppLauncher *aLauncher,
-#endif
 						   nsISupports *aWindowContext,
 						   const PRUnichar *aDefaultFile,
 						   const PRUnichar *aSuggestedFileExtension,
 						   nsILocalFile **_retval)
 {
-	nsresult rv;
-
 	mContext = aWindowContext;
 
-	nsCOMPtr<nsIDOMWindowInternal> windowInternal = 
-					do_QueryInterface (aWindowContext);
-	
-	nsCOMPtr<nsILocalFile> saveDir;
+	nsCOMPtr<nsIDOMWindow> parentDOMWindow = do_GetInterface (aWindowContext);
+	GtkWidget *parentWindow = GaleonUtils::FindGtkParent (parentDOMWindow);
 
-	/* first, get the this-session-only download dir from the pref */
-	gchar *dirName = eel_gconf_get_string (CONF_STATE_LAST_DOWNLOAD_DIR);
-	if (dirName && dirName[0] == '\0')
-	{
-		g_free (dirName);
-		dirName = NULL;
-	}
-
-	/* if we don't have a dir yet, get the normal pref */
-	if (!dirName) dirName = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
-
-	/* if we still don't have a dir, use the home directory */
-	if (!dirName || dirName[0] == '\0')
-	{
-		if (dirName) g_free (dirName);
-		dirName = g_strdup (g_get_home_dir());
-	}
-
-	/* expand the tilde, if there is one */
-	gchar *expanded_directory = gul_string_expand_home_dir (dirName);
-	g_free (dirName);
-
-	/* make sure the directory exists, and use the home directory
-	 * otherwise */
-	if (!expanded_directory ||
-	    !g_file_test (expanded_directory, G_FILE_TEST_IS_DIR))
-	{
-		if (expanded_directory) g_free (expanded_directory);
-		expanded_directory = g_strdup (g_get_home_dir());
-	}
-
-	saveDir = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-	saveDir->InitWithPath (NS_ConvertUTF8toUCS2(expanded_directory));
-	g_free (expanded_directory);
-
-	nsCOMPtr <nsILocalFile> saveFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
-
-	// If we aren't asking for downloading dir, check that we aren't
-	// overwriting anything.  If we are, pop up a warning dialog and show
-	// the filepicker if the user doesn't want to overwrite the file.
-	PRInt16 okToSave = nsIFilePicker::returnCancel;
+	/* Get the directory to use for downloading into */
 	if (!eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR))
 	{
-		nsAutoString uDirName;
-		saveDir->GetPath (uDirName);
+		nsresult rv =  BuildDownloadPath (GulCString (aDefaultFile).get(),
+						  parentWindow, _retval);
+		if (NS_SUCCEEDED (rv)) return rv;
+	}
 
-		const nsACString &cDirName = NS_ConvertUCS2toUTF8(uDirName);
-		const nsACString &cDefaultFile = NS_ConvertUCS2toUTF8(aDefaultFile);
-		char *filename = 
-			g_build_filename (PromiseFlatCString(cDirName).get(),
-					  PromiseFlatCString(cDefaultFile).get(),
-					  NULL);
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
 
-		// FIXME determine the parent widget if possible
-		if (gul_gui_confirm_overwrite_file (NULL, filename))
+	GulFileChooser *dialog;
+	dialog = gul_file_chooser_new (_("Save"), parentWindow,
+					GTK_FILE_CHOOSER_ACTION_SAVE,
+					CONF_STATE_LAST_DOWNLOAD_DIR);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
+					   GulCString (aDefaultFile).get());
+
+	while (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+	{
+		gchar *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+		if (!filename)
 		{
-			saveFile->InitWithPath (uDirName + NS_LITERAL_STRING("/") +
-					        nsDependentString(aDefaultFile));
-			okToSave = nsIFilePicker::returnOK;
+			continue;
 		}
 
+		if (gul_gui_confirm_overwrite_file (GTK_WIDGET (dialog),
+						    filename) == FALSE)
+		{
+			g_free (filename);
+			continue;
+		}
+
+		nsCOMPtr <nsILocalFile> destFile;
+
+		NS_NewNativeLocalFile (GulDependentCString (filename),
+				       PR_TRUE,
+				       getter_AddRefs(destFile));
 		g_free (filename);
-	}
 
-	if (okToSave == nsIFilePicker::returnCancel)
-	{
-		nsCOMPtr<nsIFilePicker> filePicker =
-					do_CreateInstance (G_FILEPICKER_CONTRACTID);
+		NS_ADDREF (*_retval = destFile);
 
-		const nsAString &title = NS_ConvertUTF8toUCS2(_("Select the destination filename"));
+		gtk_widget_destroy (GTK_WIDGET (dialog));
 
-		filePicker->Init (windowInternal,
-				   PromiseFlatString(title).get(), 
-				   nsIFilePicker::modeSave);
-		filePicker->SetDefaultString (aDefaultFile);
-		filePicker->SetDisplayDirectory (saveDir);
-
-		filePicker->Show (&okToSave);
-
-		if (okToSave == nsIFilePicker::returnOK)
-		{
-			filePicker->GetFile (getter_AddRefs(saveFile));
-		}
-	}
-
-	if (okToSave == nsIFilePicker::returnCancel)
-		return NS_ERROR_FAILURE;
-	else
-	{
-		nsCOMPtr<nsIFile> directory;
-		rv = saveFile->GetParent (getter_AddRefs(directory));
-		if(NS_SUCCEEDED(rv) && directory)
-		{
-			nsAutoString uDirectoryName;
-			rv = directory->GetPath(uDirectoryName);
-			const nsACString &cDirectoryName = NS_ConvertUCS2toUTF8(uDirectoryName);
-			if(NS_SUCCEEDED(rv))
-			{
-				eel_gconf_set_string (CONF_STATE_LAST_DOWNLOAD_DIR,
-						      PromiseFlatCString(cDirectoryName).get());
-			}
-		}
-
-		NS_IF_ADDREF (*_retval = saveFile);
 		return NS_OK;
 	}
-}
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 
-#if MOZILLA_SNAPSHOT <= 9
-/* void showProgressDialog (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
-NS_METHOD GContentHandler::ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
-					      nsISupports *aContext)
-{
-	g_print ("GContentHandler::ShowProgressDialog is depreciated!\n");
-	return NS_ERROR_NOT_IMPLEMENTED;
+	return NS_ERROR_FAILURE;
 }
-#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 // begin local public methods impl
@@ -444,7 +373,7 @@ NS_METHOD GContentHandler::LaunchExternalDownloader (void)
 
 NS_METHOD GContentHandler::FindHelperApp (void)
 {
-	if (mUrlHelper)
+	if (mUrlHelper || mScheme.Equals("file"))
 	{
 		return LaunchHelperApp ();
 	}
@@ -463,7 +392,7 @@ NS_METHOD GContentHandler::FindHelperApp (void)
 
 NS_METHOD GContentHandler::LaunchHelperApp (void)
 {
-	if (mMimeType)
+	if (mMimeType.Length())
 	{
 		nsresult rv;
 		nsCOMPtr<nsIExternalHelperAppService> helperService =
@@ -476,26 +405,22 @@ NS_METHOD GContentHandler::LaunchHelperApp (void)
 			appLauncher->DeleteTemporaryFileOnExit(mTempFile);
 		}
 
-		nsString uFileName;
-		mTempFile->GetPath(uFileName);
-		const nsCString &aFileName = NS_ConvertUCS2toUTF8(uFileName);
+		GulCString filename;
+		mUri->GetPath(filename);
 
-		const nsCString &document = (mUrlHelper) ? mUrl : aFileName;
+		/* FIXME: check return value */
+		gul_general_launch_application (mHelperApp,
+						mUrlHelper ? mUrl.get()
+							   : filename.get(),
+						gtk_get_current_event_time());
 
-		char *param = g_strdup (document.get());
-		gul_general_launch_application (mHelperApp->command,
-						param,
-						mHelperApp->requires_terminal);
-
-		if(mUrlHelper) mLauncher->Cancel();
-
-		g_free (param);
-	}
-	else
-	{
-		mLauncher->Cancel ();
 	}
 
+#ifdef HAVE_NSICANCELABLE_H
+	mLauncher->Cancel (NS_BINDING_ABORTED);
+#else
+	mLauncher->Cancel ();
+#endif
 	return NS_OK;
 }
 
@@ -511,32 +436,45 @@ NS_METHOD GContentHandler::GetContext (nsISupports * *_retval)
 	return NS_OK;
 }
 
-static gboolean 
-application_support_scheme (GnomeVFSMimeApplication *app, const nsCString &aScheme)
-{
-	GList *l;
 
-	g_return_val_if_fail (app != NULL, FALSE);
-	g_return_val_if_fail (!aScheme.IsEmpty(), FALSE);
+NS_METHOD GContentHandler::CheckAppSupportScheme (void)
+{
+	g_return_val_if_fail (mHelperApp, NS_ERROR_FAILURE);
+
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+
+	/* There is no way to get the supported schemes in the new API */
+	mUrlHelper = gnome_vfs_mime_application_supports_uris (mHelperApp);
+#else
+	mUrlHelper = FALSE;
+
+	if (mHelperApp->expects_uris != GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS)
+	{
+		return NS_OK;
+	}
 	
-	if (app->expects_uris != GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS)
-		return FALSE;
-	
-	for (l = app->supported_uri_schemes; l != NULL; l = l->next)
+	GList *l;
+	for (l = mHelperApp->supported_uri_schemes; l != NULL; l = l->next)
 	{
 		char *uri_scheme = (char *)l->data;
 		g_return_val_if_fail (uri_scheme != NULL, FALSE);
-		if (aScheme.Equals(uri_scheme)) return TRUE;
+		if (mScheme.Equals (uri_scheme))
+		{
+			mUrlHelper = TRUE;
+			return NS_OK;
+		}
 	}
+#endif
 
-	return FALSE;
+	return NS_OK;
 }
 
 NS_METHOD GContentHandler::SetHelperApp(GnomeVFSMimeApplication *aHelperApp,
 					PRBool alwaysUse)
 {
 	mHelperApp = aHelperApp;
-	mUrlHelper = application_support_scheme (aHelperApp, mScheme);
+
+	CheckAppSupportScheme();
 
 	return NS_OK;
 }
@@ -548,26 +486,32 @@ NS_METHOD GContentHandler::SynchroniseMIMEInfo (void)
 	rv = mLauncher->GetMIMEInfo(getter_AddRefs(mimeInfo));
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsILocalFile> helperFile;
-	rv = NS_NewNativeLocalFile(nsDependentCString(mHelperApp->command),
-				   PR_TRUE,
-				   getter_AddRefs(helperFile));
+	// XXX information passing kludge (to ProgressListener)
+	// TODO: should probably use some prefix, like 'gnomevfs:'
+	
+	const char * id;
+#ifdef HAVE_NEW_GNOME_VFS_MIME_API
+	id = gnome_vfs_mime_application_get_desktop_id (mHelperApp);
+#else
+	id = mHelperApp->id;
+#endif
+
+	/* The current time is fine here as the user has just clicked
+	 * a button (it is used as the time for the application opening) */
+	char * info = g_strdup_printf ("%d:%s", gtk_get_current_event_time(),
+				       id);
+
+	rv = mimeInfo->SetApplicationDescription
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
+		(GulString(info));
+#else
+		(GulString(info).get());
+#endif
+	g_free (info);
+
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-	rv = mimeInfo->SetPreferredApplicationHandler(helperFile);
-	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;	
-
-	nsMIMEInfoHandleAction mimeInfoAction;
-	mimeInfoAction = nsIMIMEInfo::useHelperApp;
-
-	if(mHelperApp->requires_terminal) //Information passing kludge!
-	{
-		rv = mimeInfo->SetApplicationDescription
-				(NS_LITERAL_STRING("runInTerminal").get());
-		if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
-	}
-
-	rv = mimeInfo->SetPreferredAction(mimeInfoAction);
+	rv = mimeInfo->SetPreferredAction(nsIMIMEInfo::useHelperApp);
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
 	return NS_OK;
@@ -582,20 +526,21 @@ NS_METHOD GContentHandler::Init (void)
 
 	nsCOMPtr<nsIMIMEInfo> MIMEInfo;
 	rv = mLauncher->GetMIMEInfo (getter_AddRefs(MIMEInfo));
-	rv = MIMEInfo->GetMIMEType (&mMimeType);
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
+	rv = MIMEInfo->GetMIMEType (mMimeType);
+#else
+	char *type = nsnull;
+	rv = MIMEInfo->GetMIMEType (&type);
+	mMimeType = type;
+	nsMemory::Free (type);
+#endif
 	
-	rv = mLauncher->GetDownloadInfo(getter_AddRefs(mUri),
-					&mTimeDownloadStarted,
-					getter_AddRefs(mTempFile));
+	rv = mLauncher->GetSource(getter_AddRefs(mUri));
+	rv = mLauncher->GetTargetFile(getter_AddRefs(mTempFile));
+
 	rv = mUri->GetSpec (mUrl);
 	rv = mUri->GetScheme (mScheme);
-#if 0
-	/* GetSource seems redundant and isn't in 0.9 This code is here while
-	   it remains unclear what GetSource is for. --phil */
-	nsCOMPtr<nsIURI> uri;
-	rv = mLauncher->GetSource(getter_AddRefs(uri));
-	rv = uri->GetSpec (mUrl);
-#endif	
+
 	ProcessMimeInfo ();
 
 	return NS_OK;
@@ -603,21 +548,30 @@ NS_METHOD GContentHandler::Init (void)
 
 NS_METHOD GContentHandler::ProcessMimeInfo (void)
 {
-	if (mMimeType == NULL ||
-	    !nsCRT::strcmp(mMimeType, "application/octet-stream"))
+	if (mMimeType.IsEmpty() ||
+	    mMimeType.Equals("application/octet-stream"))
 	{
-		nsresult rv;
-		nsCOMPtr<nsIURL> url = do_QueryInterface(mUri, &rv);
-		if (NS_SUCCEEDED(rv) && url)
-		{
-			nsCAutoString uriFileName;
-			url->GetFileName(uriFileName);
-			mMimeType = g_strdup
-					(gnome_vfs_mime_type_from_name
-						(uriFileName.get()));
-		}
-		else
-			mMimeType = g_strdup ("application/octet-stream");
+		// suggested filename
+		GulCString uriFileName;
+
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
+		GulString filename;
+		mLauncher->GetSuggestedFileName (filename);
+		uriFileName = filename;
+#else
+		PRUnichar *filename = nsnull;
+		mLauncher->GetSuggestedFileName(&filename);
+		uriFileName = filename;
+		nsMemory::Free(filename);
+#endif
+
+#ifdef HAVE_GNOME_VFS_GET_MIME_TYPE_FOR_NAME
+		mMimeType.Assign(gnome_vfs_get_mime_type_for_name
+				 (uriFileName.get()));
+#else
+		mMimeType.Assign(gnome_vfs_mime_type_from_name
+				 (uriFileName.get()));
+#endif
 	}
 
 	return NS_OK;
@@ -625,10 +579,23 @@ NS_METHOD GContentHandler::ProcessMimeInfo (void)
 
 NS_METHOD GContentHandler::MIMEAskAction (void)
 {
-	nsCOMPtr<nsIDOMWindow> parent = do_QueryInterface(mContext);
-	GtkWidget *parentWidget = MozillaFindGtkParent(parent);
+	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface(mContext);
+	GtkWidget *parentWidget = GaleonUtils::FindGtkParent(parent);
 
-	new MimeAskActionDialog(this, parentWidget, mMimeType);
+	MimeAskActionDialog *dialog =
+		new MimeAskActionDialog(this, parentWidget, mMimeType.get());
+
+	/* Create a mini progress listener while the mime ask action
+	 * dialog is being displayed. Using this listener, we can catch
+	 * errors while the download is happening to a temporary file
+	 * - bug 148739
+	 */
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+	nsCOMPtr<nsIWebProgressListener2> listener = new MiniProgressListener( dialog );
+#else
+	nsCOMPtr<nsIWebProgressListener> listener = new MiniProgressListener( dialog );
+#endif
+	mLauncher->SetWebProgressListener( listener );
 
 	return NS_OK;
 }
@@ -644,31 +611,46 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 					 mParent(aParentWidget)
 {
 	GtkWidget *label;
-	const char *description;
-	char ltext[255]; //philipl: Fixed length buffer == potential security problem...
-
-	mGXml = gul_glade_widget_new("galeon.glade", "mime_ask_action_dialog", 
-				     &mDialogWidget, this);
-
-	mAppMenu = glade_xml_get_widget(mGXml, "mime_ask_dialog_app_menu");
-	mToggleButton = glade_xml_get_widget(mGXml, "mime_ask_dialog_toggle_helpers");
-	mListView = glade_xml_get_widget(mGXml, "mime_ask_dialog_helper_list_view");
-	mScrolledView = glade_xml_get_widget(mGXml, "mime_ask_dialog_scrolled_view");
-
-	cddb_disclosure_set_labels (CDDB_DISCLOSURE (mToggleButton),
-			            _("Helper Applications"),
-			            _("Helper Applications"));
+	GladeXML *GXml;
+	char *markup;
+	LOG ("MimeAskActionDialog ctor(%p)", this );
 
 	mApps = gnome_vfs_mime_get_all_applications(aMimeType);
 	if (mApps)
 	{
+		GtkListStore * liststore;
+
+		GXml = gul_glade_widget_new("galeon.glade",
+					     "mime_ask_open_dialog", 
+					     &mDialogWidget, this);
+
+		mListView = glade_xml_get_widget(GXml, "mime_ask_dialog_helper_list_view");
+
 		GnomeVFSMimeApplication *defaultApp =
 			gnome_vfs_mime_get_default_application(aMimeType);
 
-		mListStore = gtk_list_store_new(N_COLUMNS,
+		char *title;
+		if (defaultApp)
+		{
+			title = g_strdup_printf (_("Open this file with \"%s\"?"), defaultApp->name);
+		}
+		else
+		{
+			// can this actually happen?
+			title = g_strdup (_("Open this file with another application?"));
+		}
+		markup = g_strdup_printf ("<span size=\"larger\" weight=\"bold\">%s</span>", title);
+		g_free (title);
+
+		label = glade_xml_get_widget(GXml, "mime_ask_action_title");
+		gtk_label_set_markup (GTK_LABEL(label), markup);
+		g_free (markup);
+
+
+		liststore = gtk_list_store_new(N_COLUMNS,
 						G_TYPE_STRING);
 		gtk_tree_view_set_model(GTK_TREE_VIEW(mListView),
-					GTK_TREE_MODEL(mListStore));
+					GTK_TREE_MODEL(liststore));
 
 		GtkCellRenderer *renderer;
 		renderer = gtk_cell_renderer_text_new();
@@ -691,8 +673,8 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 				static_cast<GnomeVFSMimeApplication *>(i->data);
 
 			GtkTreeIter iter;
-			gtk_list_store_append (mListStore, &iter);
-			gtk_list_store_set (mListStore, &iter,
+			gtk_list_store_append (liststore, &iter);
+			gtk_list_store_set (liststore, &iter,
 					    APP_COLUMN, app->name,
 					    -1);
 
@@ -703,6 +685,8 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 			}
 		}
 
+		g_object_unref (liststore);
+
 		// Calculate the correct width for the list view for showing
 		// NUM_APPLICATION rows.
 		gint width, height, ypad;
@@ -714,41 +698,77 @@ MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
 	}
 	else
 	{
+		GXml = gul_glade_widget_new("galeon.glade",
+					     "mime_ask_save_dialog", 
+					     &mDialogWidget, this);
 		GtkWidget *w;
-		w = glade_xml_get_widget(mGXml, "mime_ask_dialog_choice_label");
-		gtk_widget_hide(w);
+		w = glade_xml_get_widget(GXml, "mime_ask_dialog_save");
+		gtk_widget_grab_default (w);
+	}
 
-		w = glade_xml_get_widget(mGXml, "mime_ask_dialog_open");
-		gtk_widget_set_sensitive(w, FALSE);
-
-		gtk_widget_set_sensitive(mToggleButton, FALSE);
-	}	
-
-	GtkWidget *mimeIcon = glade_xml_get_widget(mGXml,
+	GtkWidget *mimeIcon = glade_xml_get_widget(GXml,
 						   "mime_ask_action_icon");
-	gtk_image_set_from_file(GTK_IMAGE(mimeIcon),
-				gnome_vfs_mime_get_icon(aMimeType));						     
+	gul_gui_image_set_from_mime_type (mimeIcon, 
+			                  aMimeType, GTK_ICON_SIZE_DIALOG);
 
-	description = gnome_vfs_mime_get_description(aMimeType);
+
+	nsCOMPtr<nsIHelperAppLauncher> launcher;
+	(void)mContentHandler->GetLauncher(getter_AddRefs(launcher));
+
+	// suggested filename
+	GulCString filename;
+#ifdef HAVE_NSIMIMEINFO_NSASTRING
+	GulString uFilename;
+	launcher->GetSuggestedFileName (uFilename);
+	filename = uFilename;
+#else
+	PRUnichar *uFilename = nsnull;
+	launcher->GetSuggestedFileName(&uFilename);
+	filename = uFilename;
+	nsMemory::Free(uFilename);
+#endif
+
+	markup = g_markup_printf_escaped("<b>%s</b>", filename.get());
+
+	label = glade_xml_get_widget(GXml, "mime_ask_action_name");
+	gtk_label_set_markup(GTK_LABEL(label), markup);
+	g_free (markup);
+
+	// MIME type description
+	const char *description = gnome_vfs_mime_get_description(aMimeType);
 	if (!description) description = aMimeType;
 	
-	g_snprintf(ltext, 255, "<b>%s</b>", description);
-	label = glade_xml_get_widget(mGXml, "mime_ask_action_description");
-	gtk_label_set_markup(GTK_LABEL(label), ltext);
+	label = glade_xml_get_widget(GXml, "mime_ask_action_description");
+	gtk_label_set_text(GTK_LABEL(label), description);
+
+	// source
+	nsCOMPtr<nsIURI> source;
+	launcher->GetSource(getter_AddRefs(source));
+
+	GulCString spec;
+	source->Resolve(NS_LITERAL_CSTRING("."), spec);
+
+	markup = g_strdup_printf ("<i>%s</i>", spec.get());
+	label = glade_xml_get_widget(GXml, "mime_ask_action_source");
+	gtk_label_set_markup(GTK_LABEL(label), markup);
+	g_free (markup);
 
 	gtk_window_set_transient_for(GTK_WINDOW(mDialogWidget), 
 				     GTK_WINDOW(aParentWidget));
 
 	gtk_widget_show(mDialogWidget);
+
+	g_object_unref(G_OBJECT(GXml));
 }
 
 MimeAskActionDialog::~MimeAskActionDialog()
 {
+	LOG ("MimeAskActionDialog dtor(%p)", this );
+
 	if(mApps)
 		gnome_vfs_mime_application_list_free(mApps);
 
 	gtk_widget_destroy(mDialogWidget);
-	g_object_unref(G_OBJECT(mGXml));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -758,12 +778,12 @@ MimeAskActionDialog::~MimeAskActionDialog()
 extern "C" void
 mime_ask_dialog_save_clicked_cb(GtkButton *button, MimeAskActionDialog *dialog)
 {
-	gtk_widget_hide(glade_xml_get_widget(dialog->mGXml, 
-					     "mime_ask_action_dialog"));
+	gtk_widget_hide(dialog->mDialogWidget);
 
-	nsresult rv;
 	nsCOMPtr<nsIHelperAppLauncher> launcher;
-	rv = dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
+	dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
+
+	launcher->SetWebProgressListener( nsnull );
 	
  	if (eel_gconf_get_boolean(CONF_DOWNLOADING_EXTERNAL_DOWNLOADER)
  	    == DOWNLOADER_BUILTIN)
@@ -772,7 +792,11 @@ mime_ask_dialog_save_clicked_cb(GtkButton *button, MimeAskActionDialog *dialog)
 	}
 	else
 	{
+#ifdef HAVE_NSICANCELABLE_H
+		launcher->Cancel(NS_BINDING_ABORTED);
+#else
 		launcher->Cancel();
+#endif
 		dialog->mContentHandler->LaunchExternalDownloader();
 	}
 	
@@ -782,11 +806,15 @@ mime_ask_dialog_save_clicked_cb(GtkButton *button, MimeAskActionDialog *dialog)
 static void
 mime_ask_dialog_download_cancel (MimeAskActionDialog *dialog)
 {
-	nsresult rv;
 	nsCOMPtr<nsIHelperAppLauncher> launcher;
-	rv = dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
+	dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
 
+	launcher->SetWebProgressListener( nsnull );
+#ifdef HAVE_NSICANCELABLE_H
+	launcher->Cancel(NS_BINDING_ABORTED);
+#else
 	launcher->Cancel();
+#endif
 
 	delete dialog;
 }
@@ -794,9 +822,8 @@ mime_ask_dialog_download_cancel (MimeAskActionDialog *dialog)
 extern "C" void
 mime_ask_dialog_open_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 {
-	nsresult rv;
 	nsCOMPtr<nsIHelperAppLauncher> launcher;
-	rv = dialog->mContentHandler->GetLauncher (getter_AddRefs(launcher));
+	dialog->mContentHandler->GetLauncher (getter_AddRefs(launcher));
 
 	GnomeVFSMimeApplication *app(NULL);
 	if (dialog->mApps)
@@ -819,12 +846,18 @@ mime_ask_dialog_open_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 
 	if (app)
 	{
+		launcher->SetWebProgressListener( nsnull );
 		dialog->mContentHandler->SetHelperApp(app, FALSE);
 		dialog->mContentHandler->FindHelperApp();
 		delete dialog;
 	}
 	else
 	{	
+		nsresult rv;
+		AutoJSContextStack stack;
+		rv = stack.Init ();
+		if (NS_FAILED (rv)) return;
+
 		mime_ask_dialog_download_cancel(dialog);
 		galeon_embed_utils_nohandler_dialog_run(dialog->mParent);
 	}
@@ -839,19 +872,98 @@ mime_ask_dialog_cancel_clicked_cb(GtkButton *button,
 }
 
 extern "C" void
-mime_ask_dialog_toggle_helpers_toggled_cb(GtkToggleButton *button,
+mime_ask_dialog_toggle_helpers_toggled_cb(GtkExpander *expander,
+					  GParamSpec *pspec,
 					  MimeAskActionDialog *dialog)
 {
-	if (gtk_toggle_button_get_active(button))
-	{
-		gtk_widget_show(dialog->mScrolledView);
-		gtk_window_set_resizable(GTK_WINDOW(dialog->mDialogWidget),
-					 TRUE);
-	}
-	else
-	{
-		gtk_window_set_resizable(GTK_WINDOW(dialog->mDialogWidget),
-					 FALSE);
-		gtk_widget_hide(dialog->mScrolledView);
-	}
+	gboolean val = gtk_expander_get_expanded (expander);
+	gtk_window_set_resizable (GTK_WINDOW(dialog->mDialogWidget), val);
 }
+
+extern "C" void
+mime_ask_dialog_close_cb(GtkWidget *dialogWidget, MimeAskActionDialog *dialog)
+{
+	nsresult rv;
+	nsCOMPtr<nsIHelperAppLauncher> launcher;
+	rv = dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
+
+	launcher->SetWebProgressListener( nsnull );
+#ifdef HAVE_NSICANCELABLE_H
+	launcher->Cancel(NS_BINDING_ABORTED);
+#else
+	launcher->Cancel();
+#endif
+}
+
+// ------------------------------------------------------------
+
+NS_IMETHODIMP MiniProgressListener::OnStatusChange(nsIWebProgress *aWebProgress, 
+					       nsIRequest *aRequest, nsresult aStatus, 
+					       const PRUnichar *aMessage)
+{
+	if (NS_SUCCEEDED (aStatus)) return NS_OK;
+
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
+	/* When the save fails, the mozilla code will call this, and
+	 * after this point, the launcher isn't valid for some reason
+	 * so we need to display an error, and get out of this code
+	 * quickly */
+	GtkWidget *dialog = 
+		hig_alert_new (GTK_WINDOW (mDialog->mParent), 
+			       (GtkDialogFlags)0,
+			       HIG_ALERT_ERROR,
+			       ("Failed to Save File."),
+			       GulCString (aMessage).get(),
+			       GTK_STOCK_OK, GTK_RESPONSE_OK,
+			       NULL);
+
+	delete mDialog; /* Close, and delete the MIME dialog */
+	mDialog = NULL;
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	return NS_OK;
+}
+
+NS_IMETHODIMP MiniProgressListener::OnStateChange(nsIWebProgress *aWebProgress, 
+					      nsIRequest *aRequest, PRUint32 aStateFlags, 
+					      nsresult aStatus)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP MiniProgressListener::OnProgressChange(nsIWebProgress *aWebProgress, 
+						 nsIRequest *aRequest, PRInt32 aCurSelfProgress,
+						 PRInt32 aMaxSelfProgress, PRInt32 aCurTotalProgress,
+						 PRInt32 aMaxTotalProgress)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP MiniProgressListener::OnLocationChange(nsIWebProgress *aWebProgress, 
+						 nsIRequest *aRequest, nsIURI *location)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP MiniProgressListener::OnSecurityChange(nsIWebProgress *aWebProgress, 
+						 nsIRequest *aRequest, PRUint32 state)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+NS_IMETHODIMP MiniProgressListener::OnProgressChange64(nsIWebProgress *aWebProgress,
+						nsIRequest *aRequest, PRInt64 aCurSelfProgress,
+						PRInt64 aMaxSelfProgress, PRInt64 aCurTotalProgress,
+						PRInt64 aMaxTotalProgress)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+#endif
+
