@@ -28,18 +28,10 @@
 
 #include "config.h"
 #include "galeon-spinner.h"
-#include "eel-gconf-extensions.h"
-#include "prefs-strings.h"
 #include "galeon-config.h"
-#include "gul-string.h"
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtkmenu.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtksignal.h>
-#include <libgnome/gnome-macros.h>
-#include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-icon-theme.h>
+#include <gtk/gtkicontheme.h>
 #include <math.h>
 
 #define spinner_DEFAULT_TIMEOUT 100	/* Milliseconds Per Frame */
@@ -49,7 +41,7 @@ struct GaleonSpinnerDetails {
 
 	GdkPixbuf *quiescent_pixbuf;
 
-	GnomeIconTheme *icon_theme;
+	GtkIconTheme *icon_theme;
 	
 	int	max_frame;
 	int	delay;
@@ -114,7 +106,8 @@ galeon_spinner_new (void)
 {
         GtkWidget *s;
 
-        s = GTK_WIDGET (g_object_new (GALEON_SPINNER_TYPE, NULL));
+        s = GTK_WIDGET (g_object_new (GALEON_TYPE_SPINNER, 
+				      "visible-window", FALSE, NULL));
 
         return s;
 }
@@ -168,7 +161,7 @@ get_spinner_dimensions (GaleonSpinner *spinner, int *spinner_width, int* spinner
 
 /* handler for handling theme changes */
 static void
-galeon_spinner_theme_changed (GnomeIconTheme *icon_theme,
+galeon_spinner_theme_changed (GtkIconTheme *icon_theme,
 		              GaleonSpinner  *spinner)
 {
 	gtk_widget_hide (GTK_WIDGET (spinner));
@@ -182,9 +175,6 @@ galeon_spinner_init (GaleonSpinner *spinner)
 {
 	GtkWidget *widget = GTK_WIDGET (spinner);
 	
-	
-	GTK_WIDGET_UNSET_FLAGS (spinner, GTK_NO_WINDOW);
-
 	gtk_widget_set_events (widget, 
 			       gtk_widget_get_events (widget)
 			       | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
@@ -193,7 +183,7 @@ galeon_spinner_init (GaleonSpinner *spinner)
 	spinner->details = g_new0 (GaleonSpinnerDetails, 1);
 	
 	spinner->details->delay = spinner_DEFAULT_TIMEOUT;		
-	spinner->details->icon_theme = gnome_icon_theme_new ();
+	spinner->details->icon_theme = gtk_icon_theme_get_default ();
 	g_signal_connect (spinner->details->icon_theme, "changed",
 			  G_CALLBACK (galeon_spinner_theme_changed), spinner);
 	
@@ -353,7 +343,7 @@ galeon_spinner_expose (GtkWidget *widget, GdkEventExpose *event)
 	int x_offset, y_offset, width, height;
 	GdkRectangle pix_area, dest;
 
-	g_return_val_if_fail (IS_GALEON_SPINNER (widget), FALSE);
+	g_return_val_if_fail (GALEON_IS_SPINNER (widget), FALSE);
 
 	if (!GTK_WIDGET_DRAWABLE (widget)) return FALSE;
 	spinner = GALEON_SPINNER (widget);
@@ -382,8 +372,8 @@ galeon_spinner_expose (GtkWidget *widget, GdkEventExpose *event)
 	x_offset = (widget->allocation.width - width) / 2;
 	y_offset = (widget->allocation.height - height) / 2;
 
-	pix_area.x = x_offset;
-	pix_area.y = y_offset;
+	pix_area.x = x_offset + widget->allocation.x;
+	pix_area.y = y_offset + widget->allocation.y; 
 	pix_area.width = width;
 	pix_area.height = height;
 
@@ -394,7 +384,8 @@ galeon_spinner_expose (GtkWidget *widget, GdkEventExpose *event)
 	
 	gc = gdk_gc_new (widget->window);
 	gdk_draw_pixbuf (widget->window, gc, pixbuf,
-			 dest.x - x_offset, dest.y - y_offset,
+			 dest.x - x_offset - widget->allocation.x,
+			 dest.y - y_offset - widget->allocation.y,
 			 dest.x, dest.y,
 			 dest.width, dest.height,
 			 GDK_RGB_DITHER_MAX, 0, 0);
@@ -438,21 +429,21 @@ galeon_spinner_start (GaleonSpinner *spinner)
 	}
 
 	if (spinner->details->timer_task != 0) {
-		gtk_timeout_remove (spinner->details->timer_task);
+		g_source_remove (spinner->details->timer_task);
 	}
 	
 	/* reset the frame count */
 	spinner->details->current_frame = 0;
-	spinner->details->timer_task = gtk_timeout_add (spinner->details->delay,
-							 bump_spinner_frame,
-							 spinner);
+	spinner->details->timer_task = g_timeout_add (spinner->details->delay,
+						      bump_spinner_frame,
+						      spinner);
 }
 
 static void
 galeon_spinner_remove_update_callback (GaleonSpinner *spinner)
 {
 	if (spinner->details->timer_task != 0) {
-		gtk_timeout_remove (spinner->details->timer_task);
+		g_source_remove (spinner->details->timer_task);
 	}
 	
 	spinner->details->timer_task = 0;
@@ -538,12 +529,13 @@ extract_frame (GdkPixbuf *grid_pixbuf, int x, int y, int size, int wanted_size)
 static void
 galeon_spinner_load_images (GaleonSpinner *spinner)
 {
-	char      *path;
+	const char *path;
 	GdkPixbuf *pixbuf;
 	int        grid_width, grid_height;
 	int        wanted_size, size;
 	int        x, y;
 	GList     *image_list;
+	GtkIconInfo *icon_info;
 	
 	galeon_spinner_unload_images (spinner);
 	
@@ -553,16 +545,28 @@ galeon_spinner_load_images (GaleonSpinner *spinner)
 	 */
 	wanted_size = spinner->details->small_mode ? 24 : 32;
 
-	path = gnome_icon_theme_lookup_icon (spinner->details->icon_theme,
-			                     "gnome-spinner", 
-					     wanted_size, NULL, &size);
-	if (path == NULL) {
+#if 0
+	/* Get the size of the toolbar */
+	gtk_icon_size_lookup (spinner->details->small_mode ? 
+			      GTK_ICON_SIZE_SMALL_TOOLBAR : 
+			      GTK_ICON_SIZE_LARGE_TOOLBAR,
+			      &wanted_size, &wanted_size);
+#endif
+
+	icon_info = gtk_icon_theme_lookup_icon (spinner->details->icon_theme,
+						"gnome-spinner", 
+						wanted_size, 0);
+	if (icon_info == NULL) {
 		g_warning ("Spinner animation not found");
 		return;
 	}
 
+	size = gtk_icon_info_get_base_size (icon_info);
+	path = gtk_icon_info_get_filename (icon_info);
+	g_return_if_fail (path != NULL);
+
 	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
-	g_free (path);
+	gtk_icon_info_free (icon_info);
 
 	grid_width  = gdk_pixbuf_get_width (pixbuf);
 	grid_height = gdk_pixbuf_get_height (pixbuf);
@@ -585,17 +589,15 @@ galeon_spinner_load_images (GaleonSpinner *spinner)
 
 	g_object_unref (pixbuf);
 
-	path = gnome_icon_theme_lookup_icon (spinner->details->icon_theme,
-			                     "gnome-spinner-rest", 
-					     wanted_size, NULL, &size);
-	if (path == NULL) {
+	pixbuf = gtk_icon_theme_load_icon (spinner->details->icon_theme,
+					   "gnome-spinner-rest", 
+					   wanted_size, 0, NULL);
+
+	if (pixbuf == NULL) {
 		g_warning ("Spinner rest icon not found");
 		return;
 	}
 
-	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
-	g_free (path);
-
 	spinner->details->quiescent_pixbuf = 
 		scale_to_real_size (pixbuf, wanted_size);
 
@@ -719,8 +721,10 @@ galeon_spinner_finalize (GObject *object)
 	galeon_spinner_remove_update_callback (spinner);
 	galeon_spinner_unload_images (spinner);
 	
-	g_object_unref (spinner->details->icon_theme);
-	
+	g_signal_handlers_disconnect_matched (spinner->details->icon_theme,
+					      G_SIGNAL_MATCH_DATA, 
+					      0, 0, NULL, NULL, object);
+
 	g_free (spinner->details);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
